﻿using C1.Win.C1FlexGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace C1FlexGridNodeAdd
{
  public partial class Form1 : Form
  {
    private int intRowParent;
    public Form1()
    {
      InitializeComponent();

      this.c1FlexGrid1.Cols.Count = 1;
      this.c1FlexGrid1.Cols.Fixed = 0;

      this.c1FlexGrid1.Tree.Column = 0;
      
      this.c1FlexGrid1.Rows.Count = this.c1FlexGrid1.Rows.Fixed;
      this.c1FlexGrid1.Cols[0].StarWidth = "*";

      //"BeginUpdate" causes the error
      this.c1FlexGrid1.BeginUpdate();

      //Create root nodes:
      for (int i = 0; i < 10; i++)
      {
        
        this.c1FlexGrid1.Rows.Add();
        int intRow = this.c1FlexGrid1.Rows.Count - 1;
        this.c1FlexGrid1.Rows[intRow].IsNode = true;
        this.c1FlexGrid1.Rows[intRow].Node.Level = 0;

        this.c1FlexGrid1[intRow, 0] = "Root " + i;

        //Store row index of root node 0:
        if (i == 0)
        {
          this.intRowParent = intRow;
        }

        //Insert childs:
        for (int intChild = 0; intChild < 5; intChild++)
        {
          Node nodeChild = this.c1FlexGrid1.Rows[intRow].Node.AddNode(NodeTypeEnum.LastChild, "Child " + i + "." + intChild);
          
          //Insert Child of this child:
          for (int intChildChild = 0; intChildChild < 5; intChildChild++)
          {
            Node nodeChildChild = this.c1FlexGrid1.Rows[nodeChild.Row.Index].Node.AddNode(NodeTypeEnum.LastChild, "Child " + i + "." + intChild + "." + intChildChild);
          }
        }
      }

      this.c1FlexGrid1.EndUpdate();
    }

    /// <summary>
    /// When this button is clicked, a new child node is added to the first root node.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void button1_Click(object sender, EventArgs e)
    {
      this.c1FlexGrid1.BeginUpdate();
      //This fails => child is added at header row.
      //Node nodeChild = this.c1FlexGrid1.Rows[this.intRowParent].Node.AddNode(NodeTypeEnum.LastChild, "INSERTED");
      //MessageBox.Show(this, "Inserted node at row " + nodeChild.Row.Index);

      //Use a more complicated code to first insert a row, then initialize the "Node".
      //Unfortunately, "node.GetNode(NodeTypeEnum.LastChild)" gets the index of the last direct child of the parent node.
      //But we need the index of the last subchild at any tree level. So call a recursive function.
      int intRowInsertAfter = this.GetLastChildRowIndex(this.c1FlexGrid1.Rows[this.intRowParent].Node);

      //Get child nodes of this node recursevily:


      int intRowInserted = this.c1FlexGrid1.Rows.Insert(intRowInsertAfter + 1).Index;
      this.c1FlexGrid1.Rows[intRowInserted].IsNode = true;
      this.c1FlexGrid1.Rows[intRowInserted].Node.Data = "INSERTED";
      this.c1FlexGrid1.Rows[intRowInserted].Node.Level = this.c1FlexGrid1.Rows[this.intRowParent].Node.Level + 1;


      this.c1FlexGrid1.EndUpdate();
    }

    /// <summary>
    /// Returns the row index of the last of child the node.
    /// This is a recursive method: if the node has children, it is called for the last child of this node.
    /// </summary>
    /// <param name="node"></param>
    /// <returns></returns>
    private int GetLastChildRowIndex(Node node)
    {
      //End the recursion if the node has no children.
      if (node.Children > 0)
      {
        Node nodeLast = node.GetNode(NodeTypeEnum.LastChild);

        return GetLastChildRowIndex(nodeLast);
      }

      else
      {
        return node.Row.Index;
      }
    }
  }
}
